// Calkowanie numeryczne - metoda Monte-Carlo
// www.algorytm.org
// (c)2008 by Tomasz Lubinski

using System;

namespace metoda_monte_carlo_cs
{
	/// <summary>
	/// Metoda Monte-Carlo.
	/// </summary>
	class Metoda_Monte_Carlo
	{
		public delegate double FUNC(double x);

		private static Random rand = new Random();

		//random number from a to b
		private static double randomPoint(double a, double b) 
		{
			
			return a + rand.NextDouble() * (b-a);
		}

		/// <summary>
		/// Oblicza calke metoda Monte-Carlo w przedziale od xp do xk z dokladnoscia n dla funkcji fun
		/// </summary>
		/// <param name="xp">poczatek przedzialu calkowania</param>
		/// <param name="xk">koniec przedzialu calkowania</param>
		/// <param name="n">dokladnosc calkowania</param>
		/// <param name="func">funkcja calkowana</param>
		/// <returns>przyblizona wartosc calki</returns>
		private static double calculate(double xp, double xk, int n, FUNC func)
		{
			double calka;

			calka = 0;

			for (int i=0; i<n; i++) 
			{
				calka += func(randomPoint(xp, xk));
			}

			calka = (calka / n) * (xk - xp);

			return calka;
		}

		//funkcja dla ktorej obliczamy calke
		private static double func(double x) 
		{
			return x*x+3;
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			double xp, xk;
			int n;
			FUNC function = new FUNC(func);
		
			Console.WriteLine("Podaj poczatek przedzialu calkowania");
			xp = double.Parse(Console.ReadLine());
		
			Console.WriteLine("Podaj koniec przedzialu calkowania");
			xk = double.Parse(Console.ReadLine());

			Console.WriteLine("Podaj dokladnosc calkowania (w setkach punktow)");
			n = int.Parse(Console.ReadLine());
			n *= 100;

			Console.WriteLine("Wartosc clki wynosi w przyblizeniu " + calculate(xp, xk, n, function));
		}
	}
}
